@@ -37,11 +37,12 @@ from localstack.constants import (
     HEADER_LOCALSTACK_AUTHORIZATION,
     HEADER_LOCALSTACK_REQUEST_URL,
 )
+from localstack.http.request import get_full_raw_path
 from localstack.services.messages import Headers, MessagePayload
 from localstack.services.messages import Request as RoutingRequest
 from localstack.services.messages import Response as RoutingResponse
 from localstack.utils.aws import aws_stack
-from localstack.utils.aws.aws_responses import LambdaResponse
+from localstack.utils.aws.aws_responses import LambdaResponse, calculate_crc32
 from localstack.utils.aws.aws_stack import is_internal_call_context
 from localstack.utils.aws.request_context import RequestContextManager, get_proxy_request_for_thread
 from localstack.utils.crypto import generate_ssl_cert
@@ -52,7 +53,6 @@ from localstack.utils.server import http2_server
 from localstack.utils.serving import Server
 from localstack.utils.strings import to_bytes, to_str
 from localstack.utils.threads import start_thread
-from localstack.utils.urls import path_from_url
 
 # set up logger
 LOG = logging.getLogger(__name__)
@@ -121,7 +121,7 @@ if EXTRA_CORS_ALLOWED_ORIGINS:
     ALLOWED_CORS_ORIGINS += EXTRA_CORS_ALLOWED_ORIGINS.split(",")
 
 
-class ProxyListener(object):
+class ProxyListener:
     # List of `ProxyListener` instances that are enabled by default for all requests.
     # For inbound flows, the default listeners are applied *before* forwarding requests
     # to the backend; for outbound flows, the default listeners are applied *after* the
@@ -257,11 +257,13 @@ class ArnPartitionRewriteListener(MessageModifyingProxyListener):
         # Only handle responses for calls from external clients
         if is_internal_call_context(headers):
             return None
-        return RoutingResponse(
+        response = RoutingResponse(
             status_code=response.status_code,
             content=self._adjust_partition(response.content),
             headers=self._adjust_partition(response.headers),
         )
+        self._post_process_response_headers(response)
+        return response
 
     def _adjust_partition_in_path(self, path: str, static_partition: str = None):
         """Adjusts the (still url encoded) URL path"""
@@ -325,24 +327,34 @@ class ArnPartitionRewriteListener(MessageModifyingProxyListener):
                 partition = self._get_partition_for_region(AWS_REGION_US_EAST_1)
         return partition
 
-    def _get_partition_for_region(self, region: str) -> str:
+    @staticmethod
+    def _get_partition_for_region(region: Optional[str]) -> str:
         # Region-Partition matching is based on the "regionRegex" definitions in the endpoints.json
         # in the botocore package.
-        if region.startswith("us-gov-"):
+        if region and region.startswith("us-gov-"):
             return "aws-us-gov"
-        elif region.startswith("us-iso-"):
+        elif region and region.startswith("us-iso-"):
             return "aws-iso"
-        elif region.startswith("us-isob-"):
+        elif region and region.startswith("us-isob-"):
             return "aws-iso-b"
-        elif region.startswith("cn-"):
+        elif region and region.startswith("cn-"):
             return "aws-cn"
-        elif re.match(r"^(us|eu|ap|sa|ca|me|af)-\w+-\d+$", region):
+        elif region and re.match(r"^(us|eu|ap|sa|ca|me|af)-\w+-\d+$", region):
             return "aws"
         else:
             raise ArnPartitionRewriteListener.InvalidRegionException(
                 f"Region ({region}) could not be matched to a partition."
             )
 
+    @staticmethod
+    def _post_process_response_headers(response: RoutingResponse) -> None:
+        """Adjust potential content lengths and checksums after modifying the response."""
+        if response.headers and response.content:
+            if "Content-Length" in response.headers:
+                response.headers["Content-Length"] = str(len(to_bytes(response.content)))
+            if "x-amz-crc32" in response.headers:
+                response.headers["x-amz-crc32"] = calculate_crc32(response.content)
+
 
 # -------------------
 # BASE BACKEND UTILS
@@ -351,7 +363,7 @@ class ArnPartitionRewriteListener(MessageModifyingProxyListener):
 T = TypeVar("T", bound="RegionBackend")
 
 
-class RegionBackend(object):
+class RegionBackend:
     """Base class for region-specific backends for the different APIs.
     RegionBackend lookup methods are not thread safe."""
 
@@ -396,11 +408,17 @@ class RegionBackend(object):
 # ---------------------
 
 
-def append_cors_headers(request_headers=None, response=None):
-    # Note: Use "response is not None" here instead of "not response"!
+def append_cors_headers(
+    request_headers: Dict = None, response: Union[Response, LambdaResponse] = None
+):
+    # use this config to disable returning CORS headers entirely (more restrictive security setting)
+    if config.DISABLE_CORS_HEADERS:
+        return
+
+    # Note: Use "response is None" here instead of "not response"
     headers = {} if response is None else response.headers
 
-    # In case we have LambdaResponse copy multivalue headers to regular headers, since
+    # In case we have LambdaResponse, copy multivalue headers to regular headers, since
     # CaseInsensitiveDict does not support "__contains__" and it's easier to deal with
     # a single headers object
     if isinstance(response, LambdaResponse):
@@ -472,12 +490,11 @@ def should_enforce_self_managed_service(method, path, headers, data):
     if config.DISABLE_CUSTOM_CORS_S3 and config.DISABLE_CUSTOM_CORS_APIGATEWAY:
         return True
     # allow only certain api calls without checking origin
-    import localstack.services.edge
+    from localstack.aws.protocol.service_router import determine_aws_service_name
+    from localstack.http.adapters import create_request_from_parts
 
-    api, _ = localstack.services.edge.get_api_from_custom_rules(method, path, data, headers) or (
-        "",
-        None,
-    )
+    request = create_request_from_parts(method, path, data, headers)
+    api = determine_aws_service_name(request)
     if not config.DISABLE_CUSTOM_CORS_S3 and api == "s3":
         return False
     if not config.DISABLE_CUSTOM_CORS_APIGATEWAY and api == "apigateway":
@@ -771,7 +788,7 @@ class DuplexSocket(ssl.SSLSocket):
 ssl.SSLContext.sslsocket_class = DuplexSocket
 
 
-class GenericProxy(object):
+class GenericProxy:
     # TODO: move methods to different class?
     @classmethod
     def create_ssl_cert(cls, serial_number=None):
@@ -942,18 +959,20 @@ def get_cert_pem_file_path():
 
 def start_proxy_server(
     port,
-    bind_address=None,
+    bind_address: Union[str, List[str]] = None,
     forward_url=None,
     use_ssl=None,
     update_listener: Optional[Union[ProxyListener, List[ProxyListener]]] = None,
     quiet=False,
-    params=None,  # TODO: not being used - should be investigated/removed
     asynchronous=True,
     check_port=True,
     max_content_length: int = None,
     send_timeout: int = None,
 ):
-    bind_address = bind_address if bind_address else BIND_HOST
+    if bind_address:
+        bind_addresses = bind_address if isinstance(bind_address, List) else [bind_address]
+    else:
+        bind_addresses = [BIND_HOST]
 
     if update_listener is None:
         listeners = []
@@ -964,7 +983,7 @@ def start_proxy_server(
 
     def handler(request, data):
         parsed_url = urlparse(request.url)
-        path_with_params = path_from_url(request.url)
+        path_with_params = get_full_raw_path(request)
         method = request.method
         headers = request.headers
         headers[HEADER_LOCALSTACK_REQUEST_URL] = str(request.url)
@@ -988,7 +1007,7 @@ def start_proxy_server(
 
     result = http2_server.run_server(
         port,
-        bind_address,
+        bind_addresses=bind_addresses,
         handler=handler,
         asynchronous=asynchronous,
         ssl_creds=ssl_creds,
