@@ -16,20 +16,21 @@ def acm():
 
 @aws_provider()
 def apigateway():
-    from localstack.services.apigateway import apigateway_listener, apigateway_starter
+    from localstack.services.apigateway.provider import ApigatewayApiListener, ApigatewayProvider
 
-    return Service(
-        "apigateway",
-        listener=apigateway_listener.UPDATE_APIGATEWAY,
-        start=apigateway_starter.start_apigateway,
-    )
+    provider = ApigatewayProvider()
+    listener = ApigatewayApiListener("apigateway", MotoFallbackDispatcher(provider))
+
+    return Service("apigateway", listener=listener, lifecycle_hook=provider)
 
 
 @aws_provider()
 def cloudformation():
-    from localstack.services.cloudformation import cloudformation_starter
+    from localstack.services.cloudformation.provider import CloudformationProvider
 
-    return Service("cloudformation", start=cloudformation_starter.start_cloudformation)
+    provider = CloudformationProvider()
+
+    return Service("cloudformation", listener=AwsApiListener("cloudformation", provider))
 
 
 @aws_provider(api="config")
@@ -113,9 +114,14 @@ def firehose():
 
 @aws_provider()
 def iam():
-    from localstack.services.iam import iam_listener, iam_starter
+    from localstack.services.iam.provider import IamProvider
+    from localstack.services.moto import MotoFallbackDispatcher
 
-    return Service("iam", listener=iam_listener.UPDATE_IAM, start=iam_starter.start_iam)
+    provider = IamProvider()
+    return Service(
+        "iam",
+        listener=AwsApiListener("iam", MotoFallbackDispatcher(provider)),
+    )
 
 
 @aws_provider()
@@ -256,21 +262,44 @@ def secretsmanager():
 
 @aws_provider()
 def ses():
-    from localstack.services.ses import ses_listener, ses_starter
+    from localstack.services.moto import MotoFallbackDispatcher
+    from localstack.services.ses.provider import SesProvider
 
-    return Service("ses", listener=ses_listener.UPDATE_SES, start=ses_starter.start_ses)
+    provider = SesProvider()
+    return Service(
+        "ses",
+        listener=AwsApiListener("ses", MotoFallbackDispatcher(provider)),
+        lifecycle_hook=provider,
+    )
 
 
 @aws_provider()
 def sns():
-    from localstack.services.sns import sns_listener, sns_starter
+    from localstack.aws.proxy import AwsApiListener
+    from localstack.services.sns.provider import SnsProvider
 
-    return Service("sns", listener=sns_listener.UPDATE_SNS, start=sns_starter.start_sns)
+    provider = SnsProvider()
+
+    return Service("sns", listener=AwsApiListener("sns", provider), lifecycle_hook=provider)
 
 
 @aws_provider()
 def sqs():
-    from localstack.services.sqs import sqs_listener, sqs_starter
+    from localstack.aws.proxy import AwsApiListener
+    from localstack.services import edge
+    from localstack.services.sqs import query_api
+    from localstack.services.sqs.provider import SqsProvider
+
+    query_api.register(edge.ROUTER)
+
+    provider = SqsProvider()
+
+    return Service("sqs", listener=AwsApiListener("sqs", provider), lifecycle_hook=provider)
+
+
+@aws_provider(api="sqs", name="legacy")
+def sqs_legacy():
+    from localstack.services.sqs.legacy import sqs_listener, sqs_starter
 
     return Service(
         "sqs",
@@ -280,16 +309,6 @@ def sqs():
     )
 
 
-@aws_provider(api="sqs", name="asf")
-def sqs_asf():
-    from localstack.aws.proxy import AwsApiListener
-    from localstack.services.sqs.provider import SqsProvider
-
-    provider = SqsProvider()
-
-    return Service("sqs", listener=AwsApiListener("sqs", provider), lifecycle_hook=provider)
-
-
 @aws_provider()
 def ssm():
     from localstack.services.moto import MotoFallbackDispatcher
@@ -304,22 +323,25 @@ def ssm():
 
 @aws_provider()
 def events():
-    from localstack.services.events import events_listener, events_starter
+    from localstack.services.events.provider import EventsProvider
+    from localstack.services.moto import MotoFallbackDispatcher
 
+    provider = EventsProvider()
     return Service(
-        "events", listener=events_listener.UPDATE_EVENTS, start=events_starter.start_events
+        "events",
+        listener=AwsApiListener("events", MotoFallbackDispatcher(provider)),
     )
 
 
 @aws_provider()
 def stepfunctions():
-    from localstack.services.stepfunctions import stepfunctions_listener, stepfunctions_starter
+    from localstack.services.stepfunctions.provider import StepFunctionsApiListener
 
+    listener = StepFunctionsApiListener()
     return Service(
         "stepfunctions",
-        listener=stepfunctions_listener.UPDATE_STEPFUNCTIONS,
-        start=stepfunctions_starter.start_stepfunctions,
-        check=stepfunctions_starter.check_stepfunctions,
+        listener=listener,
+        lifecycle_hook=listener.provider,
     )
 
 
