@@ -12,7 +12,6 @@ from urllib.parse import parse_qs, parse_qsl, quote, unquote, urlencode, urlpars
 
 import botocore.config
 import dateutil.parser
-import six
 import xmltodict
 from botocore.client import ClientError
 from moto.s3.exceptions import InvalidFilterRuleName
@@ -31,6 +30,7 @@ from localstack.services.s3.s3_utils import (
     extract_key_name,
     get_forwarded_for_host,
     is_expired,
+    is_object_download_request,
     is_static_website,
     normalize_bucket_name,
     uses_host_addressing,
@@ -631,19 +631,6 @@ def append_metadata_headers(method, query_map, headers):
                 headers[key] = value[0]
 
 
-def fix_location_constraint(response):
-    """Make sure we return a valid non-empty LocationConstraint, as this otherwise breaks Serverless."""
-    try:
-        content = to_str(response.content or "") or ""
-    except Exception:
-        content = ""
-    if "LocationConstraint" in content:
-        pattern = r"<LocationConstraint([^>]*)>\s*</LocationConstraint>"
-        replace = r"<LocationConstraint\1>%s</LocationConstraint>" % aws_stack.get_region()
-        response._content = re.sub(pattern, replace, content)
-        remove_xml_preamble(response)
-
-
 def fix_range_content_type(bucket_name, path, headers, response):
     # Fix content type for Range requests - https://github.com/localstack/localstack/issues/1259
     if "Range" not in headers:
@@ -722,15 +709,27 @@ def fix_creation_date(method, path, response):
     )
 
 
-def fix_delimiter(data, headers, response):
-    if response.status_code == 200 and response._content:
-        c, xml_prefix, delimiter = response._content, "<?xml", "<Delimiter><"
-        pattern = "[<]Delimiter[>]None[<]"
-        if isinstance(c, bytes):
-            xml_prefix, delimiter = xml_prefix.encode(), delimiter.encode()
-            pattern = pattern.encode()
-        if c.startswith(xml_prefix):
-            response._content = re.compile(pattern).sub(delimiter, c)
+def replace_in_xml_response(response, search: str, replace: str):
+    if response.status_code != 200 or not response._content:
+        return
+    c, xml_prefix = response._content, "<?xml"
+    if isinstance(c, bytes):
+        xml_prefix, search, replace = xml_prefix.encode(), search.encode(), replace.encode()
+    if c.startswith(xml_prefix):
+        response._content = re.compile(search).sub(replace, c)
+
+
+def fix_delimiter(response):
+    replace_in_xml_response(response, "<Delimiter>None<", "<Delimiter><")
+
+
+def fix_xml_preamble_newline(method, path, headers, response):
+    # some tools (Serverless) require a newline after the "<?xml ...>\n" preamble line, e.g., for LocationConstraint
+    # this is required because upstream moto is generally collapsing all S3 XML responses:
+    # https://github.com/spulec/moto/blob/3718cde444b3e0117072c29b087237e1787c3a66/moto/core/responses.py#L102-L104
+    if is_object_download_request(method, path, headers):
+        return
+    replace_in_xml_response(response, r"(<\?xml [^>]+>)<", r"\1\n<")
 
 
 def convert_to_chunked_encoding(method, path, response):
@@ -818,7 +817,7 @@ def set_lifecycle(bucket_name, lifecycle):
     if not exists:
         return xml_response(body, status_code=code)
 
-    if isinstance(to_str(lifecycle), six.string_types):
+    if isinstance(to_str(lifecycle), str):
         lifecycle = xmltodict.parse(lifecycle)
     BUCKET_LIFECYCLE[bucket_name] = lifecycle
     return 200
@@ -840,7 +839,7 @@ def set_replication(bucket_name, replication):
     if not exists:
         return xml_response(body, status_code=code)
 
-    if isinstance(to_str(replication), six.string_types):
+    if isinstance(to_str(replication), str):
         replication = xmltodict.parse(replication)
     BUCKET_REPLICATIONS[bucket_name] = replication
     return 200
@@ -1107,7 +1106,7 @@ def handle_put_bucket_notification(bucket, data):
         configs = config if isinstance(config, list) else [config] if config else []
         for config in configs:
             events = config.get("Event")
-            if isinstance(events, six.string_types):
+            if isinstance(events, str):
                 events = [events]
             event_filter = config.get("Filter", {})
             # make sure FilterRule is an array
@@ -1492,14 +1491,14 @@ class ProxyListenerS3(PersistingProxyListener):
             )
             append_last_modified_headers(response=response)
             append_list_objects_marker(method, path, data, response)
-            fix_location_constraint(response)
             fix_range_content_type(bucket_name, path, headers, response)
             fix_delete_objects_response(bucket_name, method, parsed, data, headers, response)
             fix_metadata_key_underscores(response=response)
             fix_creation_date(method, path, response=response)
             ret304_on_etag(data, headers, response)
             append_aws_request_troubleshooting_headers(response)
-            fix_delimiter(data, headers, response)
+            fix_delimiter(response)
+            fix_xml_preamble_newline(method, path, headers, response)
 
             if method == "PUT":
                 set_object_expiry(path, headers)
@@ -1543,7 +1542,7 @@ class ProxyListenerS3(PersistingProxyListener):
                         response.headers[header_name] = query_map[param_name][0]
 
             if response_content_str and response_content_str.startswith("<"):
-                is_bytes = isinstance(response._content, six.binary_type)
+                is_bytes = isinstance(response._content, bytes)
                 response._content = response_content_str
 
                 append_last_modified_headers(response=response, content=response_content_str)
@@ -1555,7 +1554,7 @@ class ProxyListenerS3(PersistingProxyListener):
                 # Note: make sure to return XML docs verbatim: https://github.com/localstack/localstack/issues/1037
                 if method != "GET" or not is_object_specific_request(path, headers):
                     response._content = re.sub(
-                        r"([^\?])>\n\s*<",
+                        r"([^?])>\n\s*<",
                         r"\1><",
                         response_content_str,
                         flags=re.MULTILINE,
@@ -1623,7 +1622,7 @@ def serve_static_website(headers, path, bucket_name):
 
     website_config = s3_client.get_bucket_website(Bucket=bucket_name)
     path_suffix = website_config.get("IndexDocument", {}).get("Suffix", "").lstrip("/")
-    index_document = "%s/%s" % (path.rstrip("/"), path_suffix)
+    index_document = ("%s/%s" % (path.rstrip("/"), path_suffix)).lstrip("/")
     try:
         return respond_with_key(status_code=200, key=index_document)
     except ClientError:
